<?php
#####################################################################
# Sample script for shot desking with Trixbox CE/Asterisk
# Aastra SIP Phones R2.2.0 or better
#
# php source code
# Provided by Aastra Telecom Ltd 2008
#
# Phone supported
#    Aastra 5i series
#####################################################################

####################################################################
# Includes
####################################################################
require_once('include/config.inc.php');
require_once('include/backend.inc.php');

####################################################################
# Private functions
####################################################################

####################################################################
# Aastra_decode_HTTP_header()
#
# Returns an array
#   0 Phone Type
#   1 Phone MAC Address
#   2 Phone firmware version
#####################################################################
function Aastra_decode_HTTP_header()
{
$user_agent=$_SERVER["HTTP_USER_AGENT"];
if(stristr($user_agent,"Aastra"))
	{
	$value=preg_split("/ MAC:/",$user_agent);
	$fin=preg_split("/ /",$value[1]);
	$value[1]=preg_replace("/\-/","",$fin[0]);
	$value[2]=preg_replace("/V:/","",$fin[1]);
	}
else
	{
	$value[0]="MSIE";
	$value[1]="NA";
	$value[2]="NA";
	}

$value[3]=$_SERVER["REMOTE_ADDR"];

return($value);
}


#####################################################################
# update_config_file(extension)
# Update the config file deleting the current extension
#
# Parameters
#     extension	user extension
#####################################################################
function update_config_file($extension)
{
# Config file
$config="startup.cfg";

# Read config file
$array = @parse_ini_file($config, true);
if($array==NULL) $array=array();

# Update config file
$handle = @fopen($config, "w");
if($handle)
	{
	foreach($array as $key=>$value)
		{
		if($key!=$extension)
			{
			fputs($handle,"[".$key."]"."\n");
			fputs($handle,"mac=".$value['mac']."\n");
			fputs($handle,"ip=".$value['ip']."\n");
			fputs($handle,"model=".$value['model']."\n\n");
			}
		}
	fclose($handle);
	}
}

#####################################################################
# delete_mac(mac)
# Deletes the MAC.cfg file for the user.
#
# Parameters
#     mac		MAC address of the phone
#
#####################################################################
function delete_mac($mac)
{
# Delete MAC.cfg
@unlink("/tftpboot/".$mac.".cfg");
}

#####################################################################
# Main code
#####################################################################
# GLOBAL VARIABLES
$XML_SERVER = "http://".$_SERVER['SERVER_ADDR'].$_SERVER['SCRIPT_NAME'];

# Retrieve parameters
$extension=$_GET["extension"];
$password=$_GET["password"];
$action=$_GET["action"];

# Set content type
header("Content-Type: text/xml");

# Process action
switch($action)
	{
	case 'display':
		# Reboot
		$output = "<AastraIPPhoneTextScreen LockIn=\"yes\" destroyOnExit=\"yes\">\n";
		$output .= "<Title>REBOOT</Title>\n";
		$output .= "<Text>Reboot.</Text>\n";
		$output .= "</AastraIPPhoneTextScreen>\n";
		break;

	default:
		# Input Password
		if ($password=='')
			{
			$output = "<AastraIPPhoneInputScreen type=\"number\" password=\"yes\" LockIn=\"yes\" destroyOnExit=\"yes\">\n";
			$output .= "<Title>Logout</Title>\n";
			$output .= "<Prompt>Enter Password</Prompt>\n";
			$output .= "<URL>$XML_SERVER?extension=$extension</URL>\n";
			$output .= "<Parameter>password</Parameter>\n";
			$output .= "<Default></Default>\n";
			$output .= "</AastraIPPhoneInputScreen>\n";
			}
		else
			{
			# IF authentication fails
			if (!verify_user($extension,$password))
				{
				# Display error
				$output = "<AastraIPPhoneTextScreen LockIn=\"yes\" destroyOnExit=\"yes\">\n";
				$output .= "<Title>Authentication failed</Title>\n";
				$output .= "<Text>Wrong credentials.</Text>\n";
				$output .= "</AastraIPPhoneTextScreen>\n";
				}
			else
				{
				# Get MAC address and type of phone
				$value=Aastra_decode_HTTP_header();

				# Erase mac.cfg
				delete_mac($value[1]);

				# Update config file
				update_config_file($extension);

				# Reboot needed
				$output = "<AastraIPPhoneExecute>\n";
				$output .= "<ExecuteItem URI=\"".$XML_SERVER."?action=display"."\"/>\n";
				$output .= "<ExecuteItem URI=\"Command: FastReboot\"/>\n";
				$output .= "</AastraIPPhoneExecute>\n";
				}
			}
		break;
	}


# Display XML Object
header("Content-Type: text/xml");
header("Content-Length: ".strlen($output));
echo $output;
exit;
?>
