<?php
#####################################################################
# Sample script for self-configuration with Trixbox CE/Asterisk
# Aastra SIP Phones R2.2.0 or better
#
# php source code
# Provided by Aastra Telecom Ltd 2008
#
# Phone supported
#    Aastra 5i series
#####################################################################

#####################################################################
# Includes
#####################################################################
require_once('include/config.inc.php');
require_once('include/backend.inc.php');
require_once('phpagi/misc.php');
require_once('phpagi/phpagi-asmanager.php');

#####################################################################
# Private functions
#####################################################################

#####################################################################
# Aastra_decode_HTTP_header()
#
# Returns an array
#   0 Phone Type
#   1 Phone MAC Address
#   2 Phone firmware version
#####################################################################
function Aastra_decode_HTTP_header()
{
$user_agent=$_SERVER["HTTP_USER_AGENT"];
if(stristr($user_agent,"Aastra"))
	{
	$value=preg_split("/ MAC:/",$user_agent);
	$fin=preg_split("/ /",$value[1]);
	$value[1]=preg_replace("/\-/","",$fin[0]);
	$value[2]=preg_replace("/V:/","",$fin[1]);
	}
else
	{
	$value[0]="MSIE";
	$value[1]="NA";
	$value[2]="NA";
	}

$value[3]=$_SERVER["REMOTE_ADDR"];

return($value);
}

#####################################################################
# lookup_config_file(extension)
# Checks if extension is already in use.
#
# Parameters
#     extension	extension t check
#
# Returns 1 if extension already in use
#####################################################################
function lookup_config_file($extension)
{
$config="startup.cfg";

# Init return
$return=0;

# Read config file
$array = @parse_ini_file($config, true);

# Test MAC address
if($array[$extension]['mac']!="") $return=1;

return($return);
}

#####################################################################
# update_config_file(extension,mac,ip,model)
# Update the config file with the new extension parameters
#
# Parameters
#     extension	user extension
#     mac		MAC address of the phone
#     ip		IP address of the phone
#     model		Phone model
#####################################################################
function update_config_file($extension,$mac,$ip,$model)
{
$config="startup.cfg";

# Read config file
$array = @parse_ini_file($config, true);
if($array==NULL) $array=array();

# Update value
$array[$extension]['mac']=$mac;
$array[$extension]['ip']=$ip;
$array[$extension]['model']=$model;

# Update config file
reset($array);
$handle = @fopen($config, "w");
if($handle)
	{
	while ($v = current($array))
		{
		fputs($handle,"[".key($array)."]"."\n");
		fputs($handle,"mac=".$v['mac']."\n");
		fputs($handle,"ip=".$v['ip']."\n");
		fputs($handle,"model=".$v['model']."\n\n");
		next($array);
		}
	fclose($handle);
	}
}

#####################################################################
# create_mac(extension,mac,username,secret,callerid,model)
# Creates the MAC.cfg file for the user.
#
# Parameters
#     extension	user extension
#     mac		MAC address of the phone
#     username	SIP authname
#     secret		SIP secret
#     callerid	User CallerID
#     model		Phone model
#
#####################################################################
function create_mac($mac,$extension,$username,$secret,$callerid,$model)
{
Global $AA_PROXY_SERVER,$AA_REGISTRAR_SERVER;

$value=preg_split("/ /",$callerid);
$result=$value[0]." ".$value[1];
$result=preg_replace("/</","(",$result);
$result=preg_replace("/>/",")",$result);

# Prepare replace strings
$search=array('/\$\$AA_SIPAUTHNAME_AA\$\$/','/\$\$AA_SIPSECRET_AA\$\$/','/\$\$AA_SIPUSERNAME_AA\$\$/','/\$\$AA_SIPCALLERID_AA\$\$/','/\$\$AA_PROXY_SERVER_AA\$\$/','/\$\$AA_REGISTRAR_SERVER_AA\$\$/');
$replace=array($username,$secret,$extension,$result,$AA_PROXY_SERVER,$AA_REGISTRAR_SERVER);

$read = @fopen($model.".cfg", "r");
if($read)
	{
	$write = @fopen("/tftpboot/".$mac.".cfg", "w");
	if($write)
		{
		# Create file header
		while($line=fgets($read,200))
			{
			$line = preg_replace($search, $replace, $line);
			fputs($write,$line);
			}
		fputs($write,"\n");
		fclose($write);
		}
	fclose($read);
	}
}

#####################################################################
# get_callerid(user)
#
# This function retrieves the user callerID of a user in the Asterisk
# registry (FreePBX 2.3)
#
# Parameters
#   @user		user ID
#
# Returns
#   CallerID as a string
#####################################################################
function get_callerid($user)
{
Global $ASTERISK_LOCATION;

# Try in the config file first
$sip_array = parse_ini_file($ASTERISK_LOCATION."sip_additional.conf", true);

# Extension exists?
if ($sip_array[$user]==NULL) $callerid="Unknown";
else 
	{
	# FreePBX ?
	if(strstr($sip_array[$user]['callerid'],'device')) 
		{
		# Connect to AGI
		$as = new AGI_AsteriskManager();
		$res = $as->connect();

		# Get value in the database
		$res = $as->Command('database get AMPUSER '.$user.'\/cidname');
		$line=split("\n", $res['data']);
		$cid=split(" ", $line[1]);
		$callerid= $cid[1].' '.$cid[2];

		# Disconnect properly	
		$as->disconnect();
		}
	else 
		{
		# Retrieve the value
		$value=explode(" <",$sip_array[$user]['callerid'],2);
		$callerid=$value[0];
		}
	}

# Return Caller ID
return($callerid);
}

#####################################################################
# get_username(user)
#
# This function retrieves the username of a user in the Asterisk 
# registry (FreePBX 2.3)
#
# Parameters
#   @user		user ID
#
# Returns
#   username as a string
#####################################################################
function get_username($user)
{
Global $ASTERISK_LOCATION;

# Get all the user data
$sip_array = @parse_ini_file($ASTERISK_LOCATION."sip_additional.conf", true);

# Collect data
if($sip_array[$user]['username']!='') $username=$sip_array[$user]['username'];
else
	{
	# Connect to AGI
	$as = new AGI_AsteriskManager();
	$res = $as->connect();

	# Get value in the database
	$res = $as->Command('database get DEVICE '.$user.'\/user');
	$line=split("\n", $res['data']);
	$cid=split(" ", $line[1]);
	$username= $cid[1];

	# Disconnect properly	
	$as->disconnect();
	}

# Return answer
return($username);
}

#####################################################################
# get_secret(user)
#
# This function retrieves the SIP password (secret) of a user in the 
# Asterisk configuration
#
# Parameters
#   @user		user ID
#
# Returns
#   secret as a string
#####################################################################
function get_secret($user)
{
Global $ASTERISK_LOCATION;

# Get all the user data
$sip_array = @parse_ini_file($ASTERISK_LOCATION."sip_additional.conf", true);

# Return answer
return($sip_array[$user]['secret']);
}


#####################################################################
# Main code
#####################################################################
# GLOBAL VARIABLES
$XML_SERVER = "http://".$_SERVER['SERVER_ADDR'].$_SERVER['SCRIPT_NAME'];
$AA_PROXY_SERVER = $_SERVER['SERVER_ADDR'];
$AA_REGISTRAR_SERVER = $_SERVER['SERVER_ADDR'];
$ASTERISK_LOCATION = "/etc/asterisk/";

# Retrieve parameters
$extension=$_GET["extension"];
$password=$_GET["password"];
$action=$_GET["action"];
$step=$_GET["step"];

# Set content type
header("Content-Type: text/xml");

# Reboot
if($action=="reboot")
	{
	$output = "<AastraIPPhoneExecute>\n";
	$output .= "<ExecuteItem URI=\"Command: Reset\"/>\n";
	$output .= "</AastraIPPhoneExecute>\n";
	header("Content-Length: ".strlen($output));
	echo $output;
	exit;
	}

# Input Extension
if ($extension=="")
	{
	$output = "<AastraIPPhoneInputScreen type=\"number\" LockIn=\"yes\">\n";
	$output .= "<Title>Initial startup</Title>\n";
	$output .= "<Prompt>Enter Extension</Prompt>\n";
	$output .= "<URL>$XML_SERVER</URL>\n";
	$output .= "<Parameter>extension</Parameter>\n";
	$output .= "<Default></Default>\n";
	$output .= "</AastraIPPhoneInputScreen>\n";
	header("Content-Length: ".strlen($output));
	echo $output;
	exit;
	}

# Input Password
if ($password=="")
	{
	$output = "<AastraIPPhoneInputScreen type=\"number\" password=\"yes\" LockIn=\"yes\" destroyOnExit=\"yes\">\n";
	$output .= "<Title>Initial startup</Title>\n";
	$output .= "<Prompt>Enter Password</Prompt>\n";
	$output .= "<URL>$XML_SERVER?extension=$extension</URL>\n";
	$output .= "<Parameter>password</Parameter>\n";
	$output .= "<Default></Default>\n";
	$output .= "</AastraIPPhoneInputScreen>\n";
	header("Content-Length: ".strlen($output));
	echo $output;
	exit;
	}

# IF authentication failed
if (!$userinfo = verify_user($extension, $password))
	{
	# Display error
	$output = "<AastraIPPhoneTextScreen LockIn=\"yes\" destroyOnExit=\"yes\">\n";
	$output .= "<Title>Authentication failed</Title>\n";
	$output .= "<Text>Wrong user and password.</Text>\n";
	$output .= "</AastraIPPhoneTextScreen>\n";
	header("Content-Type: text/xml");
	header("Content-Length: ".strlen($output));
	echo $output;
  	exit;
	}

# IF already configured
if(lookup_config_file($extension)==1)
	{
	# Display error
	$output = "<AastraIPPhoneTextScreen LockIn=\"yes\" destroyOnExit=\"yes\">\n";
	$output .= "<Title>Error</Title>\n";
	$output .= "<Text>Extension already in use.</Text>\n";
	$output .= "</AastraIPPhoneTextScreen>\n";
	header("Content-Type: text/xml");
	header("Content-Length: ".strlen($output));
	echo $output;
  	exit;
	}

# Get all the user data
$sip_array = parse_ini_file($ASTERISK_LOCATION."sip_additional.conf", true);

# If user not found
if ($sip_array[$extension]==NULL)
	{ 
	# Display error
	$output = "<AastraIPPhoneTextScreen LockIn=\"yes\" destroyOnExit=\"yes\">\n";
	$output .= "<Title>Internal error</Title>\n";
	$output .= "<Text>Extension is not provisioned.</Text>\n";
	$output .= "</AastraIPPhoneTextScreen>\n";
	header("Content-Type: text/xml");
	header("Content-Length: ".strlen($output));
	echo $output;
  	exit;
	}
else
	{
	# Collect data
	$username=get_username($extension);
	$secret=get_secret($extension);
	$callerid=get_callerid($extension);
	}

# Get MAC address and type of phone
$value=Aastra_decode_HTTP_header();

# Create mac.cfg
create_mac($value[1],$extension,$username,$secret,$callerid,$value[0]);

# Update config file
update_config_file($extension,$value[1],$value[3],$value[0]);

# Create Reboot screen
$output = "<AastraIPPhoneTextScreen destroyOnExit=\"yes\">\n";
$output .= "<Title>REBOOT</Title>\n";
$output .= "<Text>Reboot</Text>\n";
$output .= "</AastraIPPhoneTextScreen>\n";
header("Refresh: 1; url=".$XML_SERVER."?action=reboot");

# Display XML Object
header("Content-Type: text/xml");
header("Content-Length: ".strlen($output));
echo $output;
exit;
?>
